/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "Bomb.h"

Bomb::Bomb(sf::Texture *texture, TileMap *tab, unsigned int w, unsigned int h, unsigned int maxh, unsigned int damages, float speed)
	: ImgAnim(texture, w, maxh, false, 0.005f)
{
	// Initialise les variables
	/* TileMap */
	m_map = tab;
	/* Valeurs communes aux bombes */
	m_isAlive = true;
	/* Valeurs personnelles */
	m_w = w;
	m_h = h;
	m_maxh = maxh;
	m_speed = speed;
	m_damages = damages;
}

Bomb::~Bomb(void)
{
}

// Mise  jour de la bombe / missile (colisions)
void Bomb::Update(float timeFactor, Tank& enemyTank)
{
	// Raccourcis pour les valeurs
	float Bx = this->getPosition().x;
	float By = this->getPosition().y;
	float Bw = BOMB_W;
	float Bh = BOMB_MH;
	float Br = this->getRotation();

	// Cre des OBB pour le test de collision
	OBB tile, bomb(Bx, By + 3, Bw, Bh - 6, Br);

	// Dplacement du Sprite
	this->move(static_cast<float>(cos(PI * Br / 180) * m_speed) * timeFactor, 
		static_cast<float>(sin(PI * Br / 180) * m_speed) * timeFactor);

	// Supprime le missile si il est hors-cran
	if (Bx < 0 || Bx >= SCREEN_WIDTH ||	By < 0 || By >= SCREEN_HEIGHT)
		this->m_isAlive = false;

	// Sinon test de colision, si le missile n'est pas dj mort (= animation en lecture)
	else if (!this->IsPlaying())
	{
		// Test de colison avec l'autre tank
		if (Collision(&bomb, &enemyTank.GetOBB()))
		{
			// Donne des dgats au Tank ennemi
			enemyTank.TakeDamages(m_damages);

			// Arrte le missile et dmarre l'animation
			m_speed = 0.f;
			this->Play();
		}

		// Test de colision avec les tiles
		for (unsigned int i = static_cast<unsigned int>(((Bx / CELL_SIZE) - ((Bw / CELL_SIZE))) - 1); 
			i < static_cast<unsigned int>(((Bx / CELL_SIZE) + ((Bw / CELL_SIZE))) + 1); 
			++i)
		{
			if (i < 0) continue;
			if (i >= m_map->GetMap().size()) break;
			for (unsigned int j = static_cast<unsigned int>(((By / CELL_SIZE) - ((Bh / CELL_SIZE))) - 1); 
				j < static_cast<unsigned int>(((By / CELL_SIZE) + ((Bh / CELL_SIZE))) + 1); 
				++j)
			{
				if (j < 0) continue;
				if (j >= m_map->GetNbColumns()) break;
				else if (!m_map->GetMap().at(i).at(j).IsHoverable()) // Le tile n'est pas survolable
				{
					// Cre l'OBB de la tile  tester actuelle et de la bombe actuelle
					tile = OBB(i * CELL_SIZE + CELL_SIZE / 2, j * CELL_SIZE + CELL_SIZE / 2, CELL_SIZE, CELL_SIZE);

					// Si il y a colision, on explose la bombe
					if (Collision(&bomb, &tile))
					{
						// Donne des dgats au Tank ennemi
						m_map->AddToTileLife(i, j, - static_cast<int>(m_damages));

						// Arrte le missile et dmarre l'animation
						m_speed = 0.f;
						this->Play();
					}
				}
			}
		}
	}
}