/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "Dialog.h"

template<typename T>
sf::Vector2f toF(sf::Vector2<T> vec)
{
	return sf::Vector2f(static_cast<float>(vec.x), static_cast<float>(vec.y));
}

namespace Dialog
{
	void Dialog(sf::RenderWindow &target, const std::string &image, const sf::String &message, bool quit, int output)
	{
		// Cration de la fentre et des vnements
		sf::Event event;

		// On vite les vnements qui ne nous sont pas destins
		//while (target.pollEvent(event));

		// Chargement de l'image
		sf::Sprite triangle;
		sf::Texture triangleTex;
		bool afficherImg = false;
		if (triangleTex.loadFromFile(image))
		{
			afficherImg = true;
			triangle.setTexture(triangleTex);
		}

		// Cration du texte
		sf::Font f;
		f.loadFromFile("img/calibri.ttf");
		sf::Text text(message, f);
		text.scale(0.8f, 0.8f);
		text.setColor(sf::Color::Black);

		// On compte le nombre de retour  la ligne
		int nbRetours = 1;
		for (unsigned int i = 0; i < message.getSize(); ++i)
			if (message[i] == '\n')
				++nbRetours;

		// On place le texte en consquences
		text.move(140.f, 60.f / nbRetours);

		// Calcul de l'emplacement du bouton
		float x = text.getGlobalBounds().left + 20.f;
		float y = text.getGlobalBounds().top + text.getGlobalBounds().height + 20.f;

		// Cration du bouton
		sf::RectangleShape button(sf::Vector2f(140.f, 30.f));
		button.setFillColor(sf::Color::Transparent);
		button.setOutlineColor(sf::Color::Black);
		button.setOutlineThickness(1.f);
		button.setPosition(x, y);
		sf::Text txtButton((quit) ? ("Quitter") : ("Ok"), f);
		txtButton.scale(0.6f, 0.6f);
		txtButton.setColor(sf::Color::Black);
		txtButton.setPosition(x + 5.f, y + 2.f);

		// Cration du fond de survol
		sf::RectangleShape backgroung(sf::Vector2f(140.f, 30.f));
		backgroung.setFillColor(sf::Color(255, 0, 0, 128));

		// Cration du fond
		sf::Image bckgImg = target.capture();
		sf::Texture bckgTex; bckgTex.loadFromImage(bckgImg);
		sf::Sprite bckg(bckgTex);

		// On centre tout
		sf::Vector2f center = toF(target.getSize()) / 2.3f - sf::Vector2f(triangle.getGlobalBounds().width, triangle.getGlobalBounds().height) / 2.f;
		center.x -= 90.f;
		text.move(center);
		triangle.move(center);
		button.move(center);
		txtButton.move(center);

		// Boucle d'affichage
		bool overFlying = false;
		while (target.isOpen())
		{
			// Evnements
			while (target.pollEvent(event))
			{
				// Si on appuie sur le bouton rouge ou sur chap : on quitte
				if (event.type == sf::Event::Closed ||
					(event.type == sf::Event::KeyReleased && (event.key.code == sf::Keyboard::Return || event.key.code == sf::Keyboard::Escape)))
					return;
				
				// Si la souris survole un bouton : on met le fond
				else if (event.type == sf::Event::MouseMoved)
				{
					overFlying = true;
					if (button.getGlobalBounds().contains(toF(sf::Vector2i(event.mouseMove.x, event.mouseMove.y))))
						backgroung.setPosition(button.getPosition());
					else
						overFlying = false;
				}

				// Clic gauche : on vrifie si on est sur un bouton et on quitte la cas chant
				else if (event.type == sf::Event::MouseButtonReleased)
				{
					if (button.getGlobalBounds().contains(toF(sf::Vector2i(event.mouseButton.x, event.mouseButton.y))))
						return;
				}
			}

			// On efface l'cran
			target.clear(sf::Color(240, 240, 240)); // Beige

			// Affiche le fond
			target.draw(bckg);

			// On dessine le triangle
			if (afficherImg)
				target.draw(triangle);

			// On affiche le texte
			target.draw(text);

			// On affiche le fond de survol du bouton
			if (overFlying)
				target.draw(backgroung);

			// On affiche le bouton
			target.draw(button);
			target.draw(txtButton);

			// On affiche l'cran
			target.display();
		}

		// Si l'erreur est importante : on quitte
		if (quit)
			exit(output);
	}

	void Error(sf::RenderWindow &target, const sf::String &message, bool quit, int output)
	{
		Dialog(target, "img/erreur.png", message, quit, output);
	}

	void Information(sf::RenderWindow &target, const sf::String &message)
	{
		Dialog(target, "img/information.png", message);
	}
}
