/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "Game.h"
#include <SFML/Audio.hpp>
#include <iostream>
#include <vector>
#include "Defines.h"
#include "Tank.h"
#include "AABB.h"
#include "OBB.h"
#include "Bomb.h"
#include "TileMap.h"
#include "Dialog.h"

Game::Game(void)
{
}

Game::~Game(void)
{
}

void Game::Play(sf::RenderWindow &window)
{
	// Chargement de la musique
	sf::Music music;
	if (!music.openFromFile("sounds/music.ogg"))
         std::cerr << "ERROR: Couldn't open \"sounds\\music.ogg\" !" << std::endl;
	music.setVolume(30.f);
	music.play();

	// Variables temps
	sf::Clock MoveClock, FPSRefreshClock;
	float MoveTimeFactor;

	// Cration de la tilemap
	sf::Image imgTileset;
	imgTileset.loadFromFile("img/tileset.bmp");
	TileMap tilemap(imgTileset);

	// Cration des Tank
	sf::Texture texTank1;
	texTank1.loadFromFile("img/tank_V.png");
	sf::Texture texTank2;
	texTank2.loadFromFile("img/tank_B.png");
	sf::Texture texBomb;
	texBomb.loadFromFile("img/bomb.png");
	sf::Texture texStrafe;
	texStrafe.loadFromFile("img/strafe.png");
	Tank Tank1(&texTank1, &texStrafe, &texBomb, tilemap.GetTank1_InitPos(), tilemap); // Flches + Ctrl D + numpad 0
	Tank Tank2(&texTank2, &texStrafe, &texBomb, tilemap.GetTank2_InitPos(), tilemap); // ZQSD + A + Espace

	// Minutage de la mort
	bool death = false;
	sf::Clock deathClock;

	// Cration du HUD (affichage tte haute)
	/* Police */
	sf::Font font;
	font.loadFromFile("img/HUD.ttf");

	/* Score + bombe du Tank 1 */
	char tmpLifeT1[21] = "0";
	int lastLifeT1 = Tank1.GetLife();
	sprintf_s(tmpLifeT1, "%d", lastLifeT1);
	sf::Text ScoreT1(tmpLifeT1, font, 50U);
	ScoreT1.setPosition(2, - 28);
	ScoreT1.setColor(sf::Color::Yellow);
	sf::Sprite AvailableBombT1(texBomb);
	AvailableBombT1.setTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));
	AvailableBombT1.setOrigin(static_cast<float>(BOMB_W), static_cast<float>(BOMB_MH / 2));
	AvailableBombT1.setPosition(ScoreT1.getGlobalBounds().width + 10 + 2, BOMB_MH + 12);
	AvailableBombT1.rotate(90);
	AvailableBombT1.scale(2.f, 2.f);

	/* Score + bombe du Tank 2 */
	char tmpLifeT2[21] = "0";
	int lastLifeT2 = Tank2.GetLife();
	sprintf_s(tmpLifeT2, "%d", lastLifeT2);
	sf::Text ScoreT2(tmpLifeT2, font, 50U);
	ScoreT2.setPosition(SCREEN_WIDTH - (ScoreT2.getGlobalBounds().width + 2), - 28);
	ScoreT2.setColor(sf::Color(0, 255, 255));
	sf::Sprite AvailableBombT2(texBomb);
	AvailableBombT2.setTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));
	AvailableBombT2.setOrigin(static_cast<float>(BOMB_W), static_cast<float>(BOMB_MH / 2));
	AvailableBombT2.setPosition(SCREEN_WIDTH - (ScoreT2.getGlobalBounds().width + 10 + 2), BOMB_MH + 12);
	AvailableBombT2.rotate(90);
	AvailableBombT2.scale(2.f, 2.f);

	// Excution de la boucle principale
	while (window.isOpen())
	{
		//---------------------------------------------------------------------------------------------------
		// Traitement des vnements
		sf::Event Event;
		while (window.pollEvent(Event))
		{
			// Fentre ferme : on quitte
			if (Event.type == sf::Event::Closed || ((Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::Escape)))
				return;

			// P, on met la musique en pause
			if ((Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::P))
			{
				if (music.getStatus() == sf::Music::Playing)
					music.pause();
				else
					music.play();
			}
			
			// A : Le tank 1 mitraille
			if ((Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::A))
				Tank1.Strafe();
			// Espace : Le tank 1 tire une bombe
			if ((Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::Space))
				Tank1.ThrowBomb();

			// Ctrl Gauche : Le tank 2 mitraille
			if ((Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::RControl))
				Tank2.Strafe();
			// Numpad 0 : Le tank 2 tire une bombe
			if ((Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::Numpad0))
				Tank2.ThrowBomb();
		}
		//---------------------------------------------------------------------------------------------------
		// Gestion des touches du jeu

		/* Dplacements */

		// Rcupre le temps pass depuis le dernier tour de boucle -> pour le dplacement
		MoveTimeFactor = MoveClock.getElapsedTime().asSeconds();
		MoveClock.restart();

		// Rotation Tank 1
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Q))
			Tank1.TurnLeft(MoveTimeFactor, Tank2);
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::D))
			Tank1.TurnRight(MoveTimeFactor, Tank2);

		// Rotation Tank 2
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Left))
			Tank2.TurnLeft(MoveTimeFactor, Tank1);
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Right))
			Tank2.TurnRight(MoveTimeFactor, Tank1);

		// Dplacement Tank 1
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Z))
			Tank1.MoveForw(MoveTimeFactor, Tank2);
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::S))
			Tank1.MoveBckw(MoveTimeFactor, Tank2);

		// Dplacement Tank 2
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Up))
			Tank2.MoveForw(MoveTimeFactor, Tank1);
		if (sf::Keyboard::isKeyPressed(sf::Keyboard::Down))
			Tank2.MoveBckw(MoveTimeFactor, Tank1);
		
		// Le Tank 2 est aussi controlable au Joystick
		if (sf::Joystick::isConnected(0U))
		{
			// Joystick 1 : Le tank 2 mitraille
			if (sf::Joystick::isButtonPressed(0U, 0U))
				Tank2.Strafe();
			// Numpad 0 : Le tank 2 tire une bombe
			if (sf::Joystick::isButtonPressed(0U, 1U))
				Tank2.ThrowBomb();

			// Rotation Tank 2
			Tank2.TurnLeft(MoveTimeFactor * (- sf::Joystick::getAxisPosition(0U, sf::Joystick::Axis::X)) / 100.f, Tank1);

			// Dplacement Tank 2
			Tank2.MoveForw(MoveTimeFactor * (- sf::Joystick::getAxisPosition(0U, sf::Joystick::Axis::Y)) / 100.f, Tank1);
		}

		//---------------------------------------------------------------------------------------------------
		// Mise  jour des tanks et des bombes
		Tank1.UpdateAnim();
		Tank2.UpdateAnim();
		Tank1.UpdateBombs(MoveTimeFactor, Tank2);
		Tank2.UpdateBombs(MoveTimeFactor, Tank1);

		//---------------------------------------------------------------------------------------------------
		// Mets  jour le HUD

		// Changement de la vie du Tank 1 dans le HUD
		if (lastLifeT1 != Tank1.GetLife())
		{
			lastLifeT1 = Tank1.GetLife();
			sprintf_s(tmpLifeT1, "%d", lastLifeT1);
			ScoreT1.setString(tmpLifeT1);
			AvailableBombT1.setPosition(ScoreT1.getGlobalBounds().width + 10 + 2, BOMB_MH + 12);
		}

		// Changement de la vie du Tank 2 dans le HUD
		if (lastLifeT2 != Tank2.GetLife())
		{
			lastLifeT2 = Tank2.GetLife();
			sprintf_s(tmpLifeT2, "%d", lastLifeT2);
			ScoreT2.setString(tmpLifeT2);
			ScoreT2.setPosition(SCREEN_WIDTH - (ScoreT2.getGlobalBounds().width + 2), - 28);
			AvailableBombT2.setPosition(SCREEN_WIDTH - (ScoreT2.getGlobalBounds().width + 10 + 2), BOMB_MH + 12);
		}

		// Anime les bombes du Tank 1 du HUD
		if (Tank1.GetBombStack().size() >= 1)
			AvailableBombT1.setTextureRect(sf::IntRect(Tank1.GetBombStack().at(0).GetCurrentFrame() * BOMB_W, 0, BOMB_W, BOMB_MH));
		else AvailableBombT1.setTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));

		// Anime les bombes du Tank 2 du HUD
		if (Tank2.GetBombStack().size() >= 1)
			AvailableBombT2.setTextureRect(sf::IntRect(Tank2.GetBombStack().at(0).GetCurrentFrame() * BOMB_W, 0, BOMB_W, BOMB_MH));
		else AvailableBombT2.setTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));

		//---------------------------------------------------------------------------------------------------
		// Efface l'cran (remplissage avec du blanc)
		window.clear(sf::Color::White);

		// Affichage du tilemap
		tilemap.Draw(window);

		// Affichage du Tank et des missiles
		window.draw(Tank1);
		window.draw(Tank2);
		Tank1.DrawBombs(window);
		Tank2.DrawBombs(window);

		// Affichage du HUD
		window.draw(ScoreT1);
		if (Tank1.GetBombStack().size() >= 1)
		{
			if (Tank1.GetBombStack().at(0).IsPlaying())
				window.draw(AvailableBombT1);
		}
		else if (Tank1.GetBombStack().size() == 0) window.draw(AvailableBombT1);
		window.draw(ScoreT2);
		if (Tank2.GetBombStack().size() >= 1)
		{
			if (Tank2.GetBombStack().at(0).IsPlaying())
				window.draw(AvailableBombT2);
		}
		else if (Tank2.GetBombStack().size() == 0) window.draw(AvailableBombT2);
		
		//---------------------------------------------------------------------------------------------------
		// Vrifie que les Tanks soient en vie
		if  (lastLifeT1 == 0 && lastLifeT2 == 0)
		{
			if (deathClock.getElapsedTime().asSeconds() > 1.f)
			{
				Dialog::Information(window, "Egalit :p");
				return;
			}
		}
		if (lastLifeT1 == 0)
		{
			if (!death)
			{
				death = true;
				deathClock.restart();
			}
			else if (deathClock.getElapsedTime().asSeconds() > 1.f)
			{
				Dialog::Information(window, "Le Tank bleu a gagn !!");
				return;
			}
		}
		else if (lastLifeT2 == 0)
		{
			if (!death)
			{
				death = true;
				deathClock.restart();
			}
			else if (deathClock.getElapsedTime().asSeconds() > 1.f)
			{
				Dialog::Information(window, "Le Tank vert a gagn !!");
				return;
			}
		}
		
		//---------------------------------------------------------------------------------------------------
		// Affichage du contenu de la fentre  l'cran
		window.display();
	}
}
