/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "ImgAnim.h"

ImgAnim::ImgAnim(sf::Texture* texture, int w, int h, bool play, float interval)
{
	// Initialise les variables
	m_isPlaying = play;
	m_currentRow = 0;
	m_currentFrame = 0;
	m_interval = interval;
	m_frameWidth = w;
	m_frameHeight = h;

	// Change la texture du sprite
	this->setTexture(*texture);
	setTextureRect(sf::IntRect(0 * m_frameWidth, 0 * m_frameHeight, m_frameWidth, m_frameHeight));

	// Calcule le nb de colones et de frames
	m_nbRow = getTexture()->getSize().y / (m_frameHeight == 0 ? 1 : m_frameHeight);
	m_nbFrames = getTexture()->getSize().x / (m_frameWidth == 0 ? 1 : m_frameWidth);
}

ImgAnim::~ImgAnim(void)
{
}

// Lance l'animation
void ImgAnim::Play()
{
	m_isPlaying = true;
}

// Relance l'animation
void ImgAnim::Restart()
{
	m_isPlaying = true;
	m_currentFrame = 0;
}

// Mets en pause
void ImgAnim::Pause()
{
	m_isPlaying = false;
}

// Change la ligne actuelle
void ImgAnim::SetRow(int row)
{
	// Pas de vrification car l'utilisateur peut la faire lui mme, et peut avoir ses raisons de faire n'importe quoi ;)
	m_currentRow = row;
	m_currentFrame = 0;
}

// Change la frame actuelle
void ImgAnim::SetFrame(int frame)
{
	// Pas de vrification car l'utilisateur peut la faire lui mme, et peut avoir ses raisons de faire n'importe quoi ;)
	m_currentFrame = frame;
}

// Change l'intervalle de mise  jour
void ImgAnim::SetInterval(float interval)
{
	m_interval = interval;
}

// Mets  jour l'animation
/* retourne true si l'animation a t mise  jour */
bool ImgAnim::UpdateAnim()
{
	// Toutes les 0.2 secondes, l'anim du tank bouge
	if (m_isPlaying && m_animClock.getElapsedTime().asSeconds() >= m_interval)
	{
		m_animClock.restart();

		// Bouge l'image
		if (++m_currentFrame >= m_nbFrames)
			m_currentFrame = 0;

		// Mets  jour la sous-texture
		setTextureRect(sf::IntRect(m_currentFrame * m_frameWidth, m_currentRow * m_frameHeight, m_frameWidth, m_frameHeight));

		// Le tile a chang
		return true;
	}
	return false;
}

// Change la texture de l'animation
void ImgAnim::SetAnimTexture(sf::Texture *texture)
{
	//m_texture = texture;
	sf::Sprite::setTexture(*texture);

	// Calcule le nb de colones et de frames
	m_nbRow = getTexture()->getSize().x / m_frameHeight;
	m_nbFrames = getTexture()->getSize().y / m_frameWidth;

	UpdateAnim();
}