/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "NetGame.h"
#include <SFML/Audio.hpp>
#include <iostream>
#include <vector>
#include "Defines.h"
#include "Tank.h"
#include "AABB.h"
#include "OBB.h"
#include "Bomb.h"
#include "TileMap.h"

NetGame::NetGame(void)
{
}

NetGame::~NetGame(void)
{
}

void NetGame::SetRenderWindow(sf::RenderWindow *window)
{
	m_window = window;
}

void NetGame::Run()
{
	// Chargement de la musique
	sf::Music music;
	if (!music.openFromFile("sounds/music.ogg"))
         std::cerr << "ERROR: Couldn't open \"sounds\\music.ogg\" !" << std::endl;
	music.setVolume(30.f);
	music.play();

	// Variables temps
	sf::Clock MoveClock, FPSRefreshClock;
	float MoveTimeFactor;

	// Cration de la tilemap
	sf::Image imgTileset;
	imgTileset.loadFromFile("img/tileset.bmp");
	TileMap tilemap(imgTileset);

	// Cration des Tank
	sf::Texture texTank1;
	texTank1.loadFromFile("img/tank_V.png");
	sf::Texture texTank2;
	texTank2.loadFromFile("img/tank_B.png");
	sf::Texture texBomb;
	texBomb.loadFromFile("img/bomb.png");
	sf::Texture texStrafe;
	texStrafe.loadFromFile("img/strafe.png");
	Tank Tank1(&texTank1, &texStrafe, &texBomb, tilemap.GetTank1_InitPos(), tilemap); // Flches + Maj + numpad 0
	Tank Tank2(&texTank2, &texStrafe, &texBomb, tilemap.GetTank2_InitPos(), tilemap); // ZQSD + A + Espace

	// Demande le numro du joueur
	int nbJ = 0;
	do {
		std::cout << "Quel est votre num\x82ro ? ";
		std::cin >> nbJ;
	} while (nbJ <= 0 || nbJ > 2);

	// Cration du HUD (affichage tte haute)
	/* Police */
	sf::Font font;
	font.loadFromFile("img/HUD.ttf");

	/* Score + bombe du Tank 1 */
	char tmpLifeT1[21] = "0";
	int lastLifeT1 = Tank1.GetLife();
	sprintf_s(tmpLifeT1, "%d", lastLifeT1);
	sf::Text ScoreT1(tmpLifeT1, font, 50U);
	ScoreT1.setPosition(2, - 28);
	ScoreT1.setColor(sf::Color::Yellow);
	sf::Sprite AvailableBombT1(texBomb);
	AvailableBombT1.setTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));
	AvailableBombT1.setOrigin(static_cast<float>(BOMB_W), static_cast<float>(BOMB_MH / 2));
	AvailableBombT1.setPosition(ScoreT1.getGlobalBounds().width + 10 + 2, BOMB_MH + 12);
	AvailableBombT1.rotate(90);
	AvailableBombT1.scale(2.f, 2.f);

	/* Score + bombe du Tank 2 */
	char tmpLifeT2[21] = "0";
	int lastLifeT2 = Tank2.GetLife();
	sprintf_s(tmpLifeT2, "%d", lastLifeT2);
	sf::Text ScoreT2(tmpLifeT2, font, 50U);
	ScoreT2.setPosition(SCREEN_WIDTH - (ScoreT2.getGlobalBounds().width + 2), - 28);
	ScoreT2.setColor(sf::Color(0, 255, 255));
	sf::Sprite AvailableBombT2(texBomb);
	AvailableBombT2.setTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));
	AvailableBombT2.setOrigin(static_cast<float>(BOMB_W), static_cast<float>(BOMB_MH / 2));
	AvailableBombT2.setPosition(SCREEN_WIDTH - (ScoreT2.getGlobalBounds().width + 10 + 2), BOMB_MH + 12);
	AvailableBombT2.rotate(90);
	AvailableBombT2.scale(2.f, 2.f);

	// Excution de la boucle principale
	while (m_window->isOpen())
	{
		//---------------------------------------------------------------------------------------------------
		// Traitement des vnements
		sf::Event Event;
		while (m_window->pollEvent(Event))
		{
			// Fentre ferme : on quitte
			if (Event.type == sf::Event::Closed || ((Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::Escape)))
				return;
			// P, on arrte la musique
			if ((Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::P))
				music.stop();
			
			// Espace : Le tank 1 mitraille
			if ((nbJ == 1 && (Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::A)) ||
				(nbJ == 2 && (Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::RControl)))
			{
				sf::Packet paquet;
                paquet << 2 << this->GetID() << 3;
                Envoyer(paquet);
				if (nbJ == 1) Tank1.Strafe();
				else		  Tank2.Strafe();
			}

			// Espace : Le tank 1 tire une bombe
			if ((nbJ == 1 && (Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::Space)) ||
				(nbJ == 2 && (Event.type == sf::Event::KeyPressed) && (Event.key.code == sf::Keyboard::Numpad0)))
			{
				sf::Packet paquet;
                paquet << 2 << this->GetID() << 4;
                Envoyer(paquet);
				if (nbJ == 1) Tank1.ThrowBomb();
				else		  Tank2.ThrowBomb();
			}
		}
		//---------------------------------------------------------------------------------------------------
		// Evnements rseau
        if (IsNouveauPaquetRecu() == true)
        {
            SetPaquetLu();

            // TODO: Fonction de traitement des paquets !
            // Test rseau -> 2, ID, 3, #, #, #
            sf::Packet paquet = GetPaquet();
			SetPaquetLu();
            int x1, x2, x3;
			float x4, x5;
            paquet >> x1 >> x2 >> x3;
			
			// Si c'est une dconnexion, on retourne au menu
			if (x1 == 0 && x3 == 2)
				return;

			// Si c'est un paquet de type 2, pas de moi
			if (x1 == 2 && x2 != this->GetID())
			{
				if (x3 == 1) // Dplacement
				{
					paquet >> x4 >> x5;
					if		(nbJ == 1) Tank2.setPosition(x4, x5);
					else if (nbJ == 2) Tank1.setPosition(x4, x5);
				}
				else if (x3 == 2) // Rotation
				{
					paquet >> x4;
					if		(nbJ == 1) Tank2.setRotation(x4);
					else if (nbJ == 2) Tank1.setRotation(x4);
				}
				else if (x3 == 3) // Tire mitraille
				{
					if		(nbJ == 1) Tank2.Strafe();
					else if (nbJ == 2) Tank1.Strafe();
				}
				if (x3 == 4) // Tir bombe
				{
					if		(nbJ == 1) Tank2.ThrowBomb();
					else if (nbJ == 2) Tank1.ThrowBomb();
				}
			}
        }
		//---------------------------------------------------------------------------------------------------
		// Gestion des touches du jeu

		/* Dplacements */

		// Rcupre le temps pass depuis le dernier tour de boucle -> pour le dplacement
		MoveTimeFactor = MoveClock.getElapsedTime().asSeconds();
		MoveClock.restart();

		// Rotation du Tank
		if ((nbJ == 1 && sf::Keyboard::isKeyPressed(sf::Keyboard::Q)) || (nbJ == 2 && sf::Keyboard::isKeyPressed(sf::Keyboard::Left)))
		{
			if		(nbJ == 1) Tank1.TurnLeft(MoveTimeFactor, Tank2);
			else if (nbJ == 2) Tank2.TurnLeft(MoveTimeFactor, Tank1);
			sf::Packet paquet;
			paquet << 2 << this->GetID() << 2 << ((nbJ == 1) ? Tank1.getRotation() : Tank2.getRotation());
            Envoyer(paquet);
		}
		if ((nbJ == 1 && sf::Keyboard::isKeyPressed(sf::Keyboard::D)) || (nbJ == 2 && sf::Keyboard::isKeyPressed(sf::Keyboard::Right)))
		{
			if		(nbJ == 1) Tank1.TurnRight(MoveTimeFactor, Tank2);
			else if (nbJ == 2) Tank2.TurnRight(MoveTimeFactor, Tank1);
			sf::Packet paquet;
			paquet << 2 << this->GetID() << 2 << ((nbJ == 1) ? Tank1.getRotation() : Tank2.getRotation());
            Envoyer(paquet);
		}

		// Dplacement du Tank
		if ((nbJ == 1 && sf::Keyboard::isKeyPressed(sf::Keyboard::Z)) || (nbJ == 2 && sf::Keyboard::isKeyPressed(sf::Keyboard::Up)))
		{
			if		(nbJ == 1) Tank1.MoveForw(MoveTimeFactor, Tank2);
			else if (nbJ == 2) Tank2.MoveForw(MoveTimeFactor, Tank1);
			sf::Packet paquet;
			paquet << 2 << this->GetID() << 1 << ((nbJ == 1) ? Tank1.getPosition().x : Tank2.getPosition().x) << ((nbJ == 1) ? Tank1.getPosition().y : Tank2.getPosition().y);
            Envoyer(paquet);
		}
		if ((nbJ == 1 && sf::Keyboard::isKeyPressed(sf::Keyboard::S)) || (nbJ == 2 && sf::Keyboard::isKeyPressed(sf::Keyboard::Down)))
		{
			if		(nbJ == 1) Tank1.MoveBckw(MoveTimeFactor, Tank2);
			else if (nbJ == 2) Tank2.MoveBckw(MoveTimeFactor, Tank1);
			sf::Packet paquet;
			paquet << 2 << this->GetID() << 1 << ((nbJ == 1) ? Tank1.getPosition().x : Tank2.getPosition().x) << ((nbJ == 1) ? Tank1.getPosition().y : Tank2.getPosition().y);
            Envoyer(paquet);
		}

		//---------------------------------------------------------------------------------------------------
		// Mise  jour des tanks et des bombes
		Tank1.UpdateAnim();
		Tank2.UpdateAnim();
		Tank1.UpdateBombs(MoveTimeFactor, Tank2);
		Tank2.UpdateBombs(MoveTimeFactor, Tank1);

		//---------------------------------------------------------------------------------------------------
		// Mets  jour le HUD

		// Changement de la vie du Tank 1 dans le HUD
		if (lastLifeT1 != Tank1.GetLife())
		{
			lastLifeT1 = Tank1.GetLife();
			sprintf_s(tmpLifeT1, "%d", lastLifeT1);
			ScoreT1.setString(tmpLifeT1);
			AvailableBombT1.setPosition(ScoreT1.getGlobalBounds().width + 10 + 2, BOMB_MH + 12);
		}

		// Changement de la vie du Tank 2 dans le HUD
		if (lastLifeT2 != Tank2.GetLife())
		{
			lastLifeT2 = Tank2.GetLife();
			sprintf_s(tmpLifeT2, "%d", lastLifeT2);
			ScoreT2.setString(tmpLifeT2);
			ScoreT2.setPosition(SCREEN_WIDTH - (ScoreT2.getGlobalBounds().width + 2), - 28);
			AvailableBombT2.setPosition(SCREEN_WIDTH - (ScoreT2.getGlobalBounds().width + 10 + 2), BOMB_MH + 12);
		}

		// Anime les bombes du Tank 1 du HUD
		if (Tank1.GetBombStack().size() >= 1)
			AvailableBombT1.setTextureRect(sf::IntRect(Tank1.GetBombStack().at(0).GetCurrentFrame() * BOMB_W, 0, BOMB_W, BOMB_MH));
		else AvailableBombT1.setTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));

		// Anime les bombes du Tank 2 du HUD
		if (Tank2.GetBombStack().size() >= 1)
			AvailableBombT2.setTextureRect(sf::IntRect(Tank2.GetBombStack().at(0).GetCurrentFrame() * BOMB_W, 0, BOMB_W, BOMB_MH));
		else AvailableBombT2.setTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));

		//---------------------------------------------------------------------------------------------------
		// Efface l'cran (remplissage avec du blanc)
		m_window->clear(sf::Color::White);

		// Affichage du tilemap
		tilemap.Draw(*m_window);

		// Affichage du Tank et des missiles
		m_window->draw(Tank1);
		m_window->draw(Tank2);
		Tank1.DrawBombs(*m_window);
		Tank2.DrawBombs(*m_window);

		// Affichage du HUD
		m_window->draw(ScoreT1);
		if (Tank1.GetBombStack().size() >= 1)
		{
			if (Tank1.GetBombStack().at(0).IsPlaying())
				m_window->draw(AvailableBombT1);
		}
		else if (Tank1.GetBombStack().size() == 0) m_window->draw(AvailableBombT1);
		m_window->draw(ScoreT2);
		if (Tank2.GetBombStack().size() >= 1)
		{
			if (Tank2.GetBombStack().at(0).IsPlaying())
				m_window->draw(AvailableBombT2);
		}
		else if (Tank2.GetBombStack().size() == 0) m_window->draw(AvailableBombT2);

		// Affichage du contenu de la fentre  l'cran
		m_window->display();
	}
}
