/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "OBB.h"
#include <cmath>

// Lire le .pdf dans "../doc" pour plus d'infos

OBB::OBB(double oX, double oY, double w, double h, double angle)
	: AABB(oX,oY,w,h), angle(angle)
{
}

OBB::~OBB(void)
{
}

// Change l'angle
void OBB::SetAngle(double new_angle)
{
	angle = new_angle;
}

// Dfinie si il y a collision entre deux OBB
bool Collision(OBB* pRect1, OBB* pRect2)
{
	OBB rectA(*pRect1);
	OBB rectB(*pRect2);
	rectB.RotateAround_private(&rectA);

	if(!Collision(&rectA, &rectB.outLineRect))
		return false;

	rectB = *pRect2;
	rectA.RotateAround_private(&rectB);

	if(!Collision(&rectA.outLineRect, &rectB))
		return false;

	return true;
}

// Mises  jour des proprits de l'OBB
void OBB::UpdateOutLineRect_private()
{
	double absCosA = abs(cos( angle * DEG_TO_RAD));
	double absSinA = abs(sin( angle * DEG_TO_RAD));
	outLineRect.w = w * absCosA + h * absSinA;
	outLineRect.h = w * absSinA + h * absCosA;
	outLineRect.oX = oX;
	outLineRect.oY = oY;
}

// Mises  jour des proprits de l'OBB
void OBB::RotateAround_private(const OBB* base)
{
	double cosA = cos( -base->angle * DEG_TO_RAD);
	double sinA = sin( -base->angle * DEG_TO_RAD);
	double oXDif = oX - base->oX;
	double oYDif = oY - base->oY;
	double oXDif2 = oXDif * cosA - oYDif * sinA;
	double oYDif2 = oXDif * sinA + oYDif * cosA;
	oX = base->oX + oXDif2;
	oY = base->oY + oYDif2;
	angle -= base->angle;
	UpdateOutLineRect_private();
}