/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "Tank.h"

Tank::Tank(sf::Texture *textureTank, sf::Texture *textureStrafe, sf::Texture *textureBomb, const sf::Vector2f& pos, TileMap &map)
	: ImgAnim(textureTank, TANK_W, TANK_H), // Initialise l'animation
	m_map(map),								// Copie l'adresse du tableau de jeu
	m_moveSpeed(SPEED),						// Dfinie la vitesse de dplacement du tank
	m_rotateSpeed(SPEED_R),					// Dfinie la vitesse de ratation du tank
	m_isMovingForw(false),					// Si le tank est en train d'avancer
	m_isMovingBckw(false),					// Si le tank est en train de reculer
	m_isExploding(false),					// Si le tank est mort
	m_life(2000)							// Vie du Tank
{
	// Initialisation des pointeurs
	m_textureStrafe = textureStrafe;
	m_textureBomb =  textureBomb;

	// Origine de l'image
	this->setOrigin(TANK_W / 2, TANK_H / 2);

	// Change la position initiale
	Tank::setPosition(pos);

	// Mets  jour l'animation du tank
	UpdateAnim();

	// Charge les sons
	/* Bombes */
	if (!m_bombBuf.loadFromFile("sounds/bomb.wav"))
         std::cerr << "ERROR: Couldn't open \"sounds\\bomb.wav\" !" << std::endl;
	m_bombSnd.setBuffer(m_bombBuf);
	m_bombSnd.setVolume(100.f);
	/* Missiles */
	if (!m_missileBuf.loadFromFile("sounds/missile.wav"))
         std::cerr << "ERROR: Couldn't open \"sounds\\missile.wav\" !" << std::endl;
	m_missileSnd.setBuffer(m_missileBuf);
	m_missileSnd.setVolume(100.f);
	/* Explosion */
	if (!m_explosionBuf.loadFromFile("sounds/explosion.wav"))
         std::cerr << "ERROR: Couldn't open \"sounds\\explosion.wav\" !" << std::endl;
	m_explosionSnd.setBuffer(m_explosionBuf);
	m_explosionSnd.setVolume(100.f);
}

Tank::~Tank(void)
{
}

// Dplacement intuitif
void Tank::MoveForw(float timeFactor, Tank& enemyTank)
{
	// Animation du tank
	m_isMovingForw = true;
	UpdateAnim();

	// Bouge le tank vers l'avant en fonction de sa rotation
	Move(static_cast<float>(cos(PI * this->getRotation() / 180) * m_moveSpeed) * timeFactor,
		static_cast<float>(sin(PI * this->getRotation() / 180) * m_moveSpeed) * timeFactor
		, enemyTank);
}

void Tank::MoveBckw(float timeFactor, Tank& enemyTank)
{
	// Animation du tank
	m_isMovingBckw = true;
	UpdateAnim();

	// Bouge le tank vers l'arrire en fonction de sa rotation
	Move(- static_cast<float>(cos(PI * this->getRotation() / 180) * m_moveSpeed) * timeFactor * 1.3f,
		- static_cast<float>(sin(PI * this->getRotation() / 180) * m_moveSpeed) * timeFactor * 1.3f
		, enemyTank);
}

void Tank::TurnRight(float timeFactor, Tank& enemyTank)
{
	rotate(+ m_rotateSpeed * timeFactor, enemyTank);
}

void Tank::TurnLeft(float timeFactor, Tank& enemyTank)
{
	rotate(- m_rotateSpeed * timeFactor, enemyTank);
}

// Dplacement prcis
void Tank::Move(float x, float y, Tank& enemyTank)
{
	// Si on est mort, on bouge pas
	if (m_isExploding) return;
	
	// Variables de test pour X et Y
	float tmpX = x, tmpY = y;

	// Test si le dplacement est succeptible de sauter les murs (rcursivit)
	if (abs(tmpX) > CELL_SIZE)
	{
		Move(tmpX - CELL_SIZE, tmpY, enemyTank);
		tmpX = CELL_SIZE - tmpX;
	}
	if (abs(tmpY) > CELL_SIZE)
	{
		Move(tmpX, tmpY - CELL_SIZE, enemyTank);
		tmpY = CELL_SIZE - tmpY;
	}

	// Vrifie le hors-cran
	if (this->getGlobalBounds().left + tmpX < 0)
		tmpX -= this->getGlobalBounds().left;
	else if (this->getGlobalBounds().left + tmpX + this->getGlobalBounds().width > SCREEN_WIDTH)
		tmpX = SCREEN_WIDTH - this->getGlobalBounds().left - this->getGlobalBounds().width;
	if (this->getGlobalBounds().top + tmpY < 0)
		tmpY -= this->getGlobalBounds().top;
	else if (this->getGlobalBounds().top + tmpY + this->getGlobalBounds().height > SCREEN_HEIGHT)
		tmpY = SCREEN_HEIGHT - this->getGlobalBounds().top - this->getGlobalBounds().height;

	// Racourci l'appel aux fonctions
	float Tx = this->getPosition().x;
	float Ty = this->getPosition().y;
	float Tw = TANK_W;
	float Th = TANK_H;
	float Tr = this->getRotation();

	// Variables de test
	OBB tmpTank;
	OBB tile;

	/* Colision avec l'autre tank */

	// Test pour le dplacement en X
	tmpTank = OBB(Tx + tmpX, Ty, Tw, Th, Tr); // Calcul de la nouvelle OBB
	while (Collision(&tmpTank, &enemyTank.GetOBB()) && tmpX != 0)
	{
		// Diminue le dplacement en X
		if (tmpX > 0.f)
			tmpX -= 0.02f;
		else
			tmpX += 0.02f;

		// Si le dpacement est trop faible, on arrete
		if (abs(tmpX) <= 0.03f)
			tmpX = 0.f;

		// Calcul de la nouvelle OBB
		tmpTank = OBB(Tx + tmpX, Ty, Tw, Th, Tr);
	}

	// Test pour le dplacement en X
	tmpTank = OBB(Tx, Ty + tmpY, Tw, Th, Tr); // Calcul de la nouvelle OBB
	while (Collision(&tmpTank, &enemyTank.GetOBB()) && tmpY != 0)
	{
		// Diminue le dplacement en X
		if (tmpY > 0.f)
			tmpY -= 0.02f;
		else
			tmpY += 0.02f;

		// Si le dpacement est trop faible, on arrete
		if (abs(tmpY) <= 0.03f)
			tmpY = 0.f;

		// Calcul de la nouvelle OBB
		tmpTank = OBB(Tx, Ty + tmpY, Tw, Th, Tr);
	}

	/* Colision avec le dcor */
	for (unsigned int i = static_cast<unsigned int>(((Tx / CELL_SIZE) - ((Tw / CELL_SIZE))) - 1); 
						i < static_cast<unsigned int>(((Tx / CELL_SIZE) + ((Tw / CELL_SIZE))) + 1); 
						++i)
	{
		if (i < 0) continue;
		if (i >= m_map.GetMap().size()) break;
		for (unsigned int j = static_cast<unsigned int>(((Ty / CELL_SIZE) - ((Th / CELL_SIZE))) - 1); 
							j < static_cast<unsigned int>(((Ty / CELL_SIZE) + ((Th / CELL_SIZE))) + 1); 
							++j)
		{
			if (j < 0) continue;
			if (j >= m_map.GetMap().at(i).size()) break;
			else if (m_map.GetMap().at(i).at(j).IsBlocking()) // Le tile est bloquant
			{
				// Cre l'OBB de la tile  tester actuelle
				// TODO: Utiliser l'AABB de la Tile
				tile = OBB(i * CELL_SIZE + CELL_SIZE / 2, j * CELL_SIZE + CELL_SIZE / 2, CELL_SIZE, CELL_SIZE);

				// Test pour le dplacement en X
				tmpTank = OBB(Tx + tmpX, Ty, Tw, Th, Tr); // Calcul de la nouvelle OBB
				while (Collision(&tmpTank, &tile) && tmpX != 0.f)
				{
					// Diminue le dplacement en X
					if (tmpX > 0.f)
						tmpX -= 0.02f;
					else
						tmpX += 0.02f;

					// Si le dpacement est trop faible, on arrete
					if (abs(tmpX) <= 0.03f)
						tmpX = 0.f;

					// Calcul de la nouvelle OBB
					tmpTank = OBB(Tx + tmpX, Ty, Tw, Th, Tr);
				}

				// Test pour le dplacement en X
				tmpTank = OBB(Tx, Ty + tmpY, Tw, Th, Tr); // Calcul de la nouvelle OBB
				while (Collision(&tmpTank, &tile) && tmpY != 0.f)
				{
					// Diminue le dplacement en X
					if (tmpY > 0.f)
						tmpY -= 0.02f;
					else
						tmpY += 0.02f;

					// Si le dpacement est trop faible, on arrete
					if (abs(tmpY) <= 0.03f)
						tmpY = 0.f;

					// Calcul de la nouvelle OBB
					tmpTank = OBB(Tx, Ty + tmpY, Tw, Th, Tr);
				}
			}
		}
	}
	Sprite::move(tmpX, tmpY);
}

void Tank::rotate(float rotation, Tank& enemyTank)
{
	// Si on est mort, on bouge pas
	if (m_isExploding) return;

	// Racourci l'appel aux fonctions
	float Tx = this->getPosition().x;
	float Ty = this->getPosition().y;
	float Tw = TANK_W;
	float Th = TANK_H;
	float Tr = this->getRotation();

	// Variables de test
	float tmpRot = rotation;
	OBB tmpTank(Tx, Ty, Tw, Th, Tr + tmpRot);
	OBB tile;
	
	/* Colision avec le dcor */
	for (unsigned int i = static_cast<unsigned int>(((Tx / CELL_SIZE) - ((Tw / CELL_SIZE))) - 1); 
						i < static_cast<unsigned int>(((Tx / CELL_SIZE) + ((Tw / CELL_SIZE))) + 1); 
						++i)
	{
		if (i < 0) continue;
		if (i >= m_map.GetMap().size()) break;
		for (unsigned int j = static_cast<unsigned int>(((Ty / CELL_SIZE) - ((Th / CELL_SIZE))) - 1); 
							j < static_cast<unsigned int>(((Ty / CELL_SIZE) + ((Th / CELL_SIZE))) + 1); 
							++j)
		{
			if (j < 0) continue;
			if (j >= m_map.GetMap().at(i).size()) break;
			else if (m_map.GetMap().at(i).at(j).IsBlocking()) // Le tile n'est pas bloquant
			{
				// Cre l'OBB de la tile  tester actuelle
				tile = OBB(i * CELL_SIZE + CELL_SIZE / 2, j * CELL_SIZE + CELL_SIZE / 2, CELL_SIZE, CELL_SIZE);

				if (Collision(&tmpTank, &tile))
					tmpRot = 0.f;
			}
		}
	}
	Sprite::rotate(tmpRot);
}

// Tire une bombe
void Tank::ThrowBomb()
{
	// Si on est mort, on tire pas
	if (m_isExploding) return;

	// 1 bombe maximum, et toutes les secondes
	if (m_bombStack.size() < 1 && m_BombClock.getElapsedTime().asSeconds() >= 1.f)
	{
		m_bombSnd.play();

		m_BombClock.restart();

		m_bombStack.push_back(Bomb(m_textureBomb, &m_map, BOMB_W, BOMB_H, BOMB_MH, BOMB_DAMAGES, BOMB_SPEED));
		m_bombStack.back().setTextureRect(sf::IntRect(0, 0, BOMB_W, BOMB_MH));
		m_bombStack.back().setOrigin(static_cast<float>(BOMB_W), static_cast<float>(BOMB_MH / 2));
		m_bombStack.back().setRotation(this->getRotation());
		m_bombStack.back().setPosition(static_cast<float>(cos(PI * this->getRotation() / 180) * (TANK_W / 2 + BOMB_W)) + this->getPosition().x,
									   static_cast<float>(sin(PI * this->getRotation() / 180) * (TANK_W / 2 + BOMB_W)) + this->getPosition().y);
	}
}

// Mitrailler
void Tank::Strafe()
{
	// Si on est mort, on tire pas
	if (m_isExploding) return;


	// Pour ne pas avoir un flux continu
	if (m_strafingClock.getElapsedTime().asSeconds() >= 0.2f)
	{
		m_missileSnd.play();

		m_strafingClock.restart();
		m_strafeStack.push_back(Bomb(m_textureStrafe, &m_map, STRAFE_W, STRAFE_H, STRAFE_MH, STRAFE_DAMAGES, STRAFE_SPEED));
		m_strafeStack.back().setTextureRect(sf::IntRect(0, 0, STRAFE_W, STRAFE_MH));
		m_strafeStack.back().setOrigin(static_cast<float>(STRAFE_W), static_cast<float>(STRAFE_MH / 2));
		m_strafeStack.back().setRotation(this->getRotation());
		m_strafeStack.back().setPosition(static_cast<float>(cos(PI * this->getRotation() / 180) * (TANK_W / 2 + STRAFE_W)) + this->getPosition().x,
											static_cast<float>(sin(PI * this->getRotation() / 180) * (TANK_W / 2 + STRAFE_W)) + this->getPosition().y);
	}
}

// Prends des dommages
void Tank::TakeDamages(int dom)
{
	if (m_life - dom > 0)
		m_life -= dom;
	else
	{
		m_life = 0;
		m_isExploding = true;
	}
}

// Mise  jour de l'animation du tank
void Tank::UpdateAnim()
{
	// Tank en vie >= 25%
	if (!m_isExploding && m_life >= 500)
		SetRow(0);

	// Tank en vie <= 25 %
	else if (!m_isExploding)
	{
		if (GetCurrentRow() != 1)
			SetRow(1);
		Play();
	}

	// Tank en train d'exploser
	else
	{
		if (GetCurrentRow() != 2)
			SetRow(2);
		Play();
	}

	ImgAnim::UpdateAnim();
}

// Mise  jour des bombes
void Tank::UpdateBombs(float timeFactor, Tank& enemyTank)
{
	// Gre la mitraille
	for (unsigned int i = 0; i < m_strafeStack.size(); ++i)
	{
		// Mets  jour la mitraille
		m_strafeStack.at(i).Update(timeFactor, enemyTank);

		// Si la mitraille est morte
		if (!m_strafeStack.at(i).m_isAlive)
		{
			m_strafeStack.erase(m_strafeStack.begin() + i);
			continue;
		}

		// Si on est arriv au bout de l'anim, on dtruit la mitraille
		if (m_strafeStack.at(i).GetCurrentFrame() == m_strafeStack.at(i).GetNbFrames() - 1)
		{
			m_strafeStack.erase(m_strafeStack.begin() + i);
			continue;
		}

		// On mets  jour l'anim
		m_strafeStack.at(i).UpdateAnim();
	}
	
	// Gre les bombes
	for (unsigned int i = 0; i < m_bombStack.size(); ++i)
	{
		// Mets  jour la bombe
		m_bombStack.at(i).Update(timeFactor, enemyTank);

		// Si la bombe est morte
		if (!m_bombStack.at(i).m_isAlive)
		{
			m_bombStack.erase(m_bombStack.begin() + i);
			continue;
		}

		// Si on est arriv au bout de l'anim, on dtruit la bombe
		if (m_bombStack.at(i).GetCurrentFrame() == m_bombStack.at(i).GetNbFrames() - 1)
		{
			m_bombStack.erase(m_bombStack.begin() + i);
			m_explosionSnd.play();
			continue;
		}

		// On mets  jour l'anim
		m_bombStack.at(i).UpdateAnim();
	}
}

// Affichage des bombes
void Tank::DrawBombs(sf::RenderTarget &target)
{
	for (unsigned int i = 0; i < m_strafeStack.size(); ++i)
		target.draw(m_strafeStack.at(i));
	for (unsigned int i = 0; i < m_bombStack.size(); ++i)
		target.draw(m_bombStack.at(i));
}
