/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#pragma once
#include <SFML\Audio.hpp>
#include <SFML\Graphics.hpp>
#include <string>
#include <vector>
#include "Defines.h"
#include "Bomb.h"
#include "OBB.h"
#include "TileMap.h"

class Bomb;

class Tank : public ImgAnim
{
public:
	Tank(sf::Texture *textureTank, sf::Texture *textureStrafe, sf::Texture *textureBomb, const sf::Vector2f& pos, TileMap &map);
	virtual ~Tank(void);

	// Dplacement intuitif
	void MoveForw(float timeFactor, Tank& enemyTank);
	void MoveBckw(float timeFactor, Tank& enemyTank);
	void TurnRight(float timeFactor, Tank& enemyTank);
	void TurnLeft(float timeFactor, Tank& enemyTank);

	// Dplacement prcis
	void Move(float x, float y, Tank& enemyTank);
	void rotate(float rotation, Tank& enemyTank);

	// Tire de la mitraille ou lance une bombe
	void Strafe(); // Mitrailler
	void ThrowBomb();

	// Prends des dommages
	void TakeDamages(int dom);

	// Mises  jour
	void UpdateAnim();
	void UpdateBombs(float timeFactor, Tank& enemyTank);

	// Dessine les bombes sur une surface de rendu
	void DrawBombs(sf::RenderTarget &target);

	// Fonctions Get
	int GetLife() const { return m_life; }
	float GetMoveSpeed() const { return m_moveSpeed; }
	float GetRotateSpeed() const { return m_rotateSpeed; }
	const sf::Texture *GetTextureBomb() const { return m_textureBomb; }
	const sf::Texture *GetTextureStrafe() const { return m_textureStrafe; }
	const std::vector<Bomb>& GetBombStack() const { return m_bombStack; }
	OBB GetOBB() const { return OBB(getPosition().x, getPosition().y, TANK_W, TANK_H, getRotation()); }

	// Fonctions Is
	bool IsMovingForw() const { return m_isMovingForw; }
	bool IsMovingBckw() const { return m_isMovingBckw; }

private:
	// Proprits du tank
	int m_life;
	bool m_isExploding;
	float m_moveSpeed;
	float m_rotateSpeed;

	// Tableau de jeu pour les colisions
	TileMap &m_map;

	// Dplacement
	bool m_isMovingForw; // Est en train d'avancer
	bool m_isMovingBckw; // Est en train de reculer

	// Bombes et mitraille
	sf::Texture *m_textureBomb;
	sf::Texture *m_textureStrafe;
	sf::Clock m_BombClock;
	std::vector<Bomb> m_bombStack;
	sf::Clock m_strafingClock;
	std::vector<Bomb> m_strafeStack;

	// Sons
	sf::SoundBuffer m_explosionBuf;
	sf::Sound m_explosionSnd;
	sf::SoundBuffer m_missileBuf;
	sf::Sound m_missileSnd;
	sf::SoundBuffer m_bombBuf;
	sf::Sound m_bombSnd;
};

