/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "Tile.h"
#include <iostream>

Tile::Tile(sf::Texture *texture)
	: ImgAnim(texture, CELL_SIZE, CELL_SIZE, false)
{
	// Vie (2 bombes ou 40 missiles)
	m_life = 300;

	// Proprits par dfaut (tile vide)
	m_isBlocking = false;
	m_isBreakable = false;
	m_isHoverable = true;
}

Tile::~Tile(void)
{
}

// Change la vie et l'tat (morte) si besoin de la tile
void Tile::SetLife(int life)
{
	// Si le tile est cassable
	if (m_isBreakable)
		m_life = life;
	UpdateAnim();
}

// Mise  jour de l'anim de la Tile (masquage de celle de ImgAnim)
bool Tile::UpdateAnim()
{
	// Si la tile est morte
	if (m_life <= 0)
	{
		// Stop si la 3nde est finie | Cendres fin fini -> Stop
		if (GetCurrentRow() == 3 && GetCurrentFrame() == GetNbFrames()-1)
		{
			// Arrte l'anim
			Pause();

			// Le tile devient comme vide
			m_isBlocking = false;
			m_isHoverable = true;
		}
		// Passe  la 3e ligne si la 2nde est finie | Cendres fini -> Cendres fin
		if (GetCurrentRow() == 2 && GetCurrentFrame() == GetNbFrames()-1)
		{
			// Change la ligne
			SetRow(3);
			Play();

			// Le tile devient comme vide
			m_isBlocking = false;
			m_isHoverable = true;
		}
		// Passe  la 2e ligne si la 3ere est finie -> Brle fini -> Cendres
		else if (GetCurrentRow() == 1 && GetCurrentFrame() == GetNbFrames()-1)
		{
			// Change la ligne
			SetRow(2);
			Play();

			// Le tile devient comme vide
			m_isBlocking = false;
			m_isHoverable = true;
		}
		else if (GetCurrentRow() == 0) // Normal -> Brle
		{
			// Dmarre l'animation
			SetRow(1);
			Play();
		}
		ImgAnim::UpdateAnim();
		return true; // L'anim a t mise  jour
	}

	// Sinon, si la tile n'est pas morte
	ImgAnim::UpdateAnim();
	return false; // L'anim n'a pas chang
}