/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "TileMap.h"

TileMap::TileMap(const sf::Image &tilemap)
	: m_tilemap(tilemap)
{
	// Calcule le nb de lignes et colonnes fait la carte
	m_nbRows = SCREEN_WIDTH / CELL_SIZE;
	m_nbColumns = SCREEN_HEIGHT / CELL_SIZE;

	// TODO: C'est pas beau ! -> Systme automatique et gestion des [ho/e]rreurs
	// Chargement des textures
	m_textures.push_back(sf::Texture());
	m_textures.back().loadFromFile("img/ground.jpg");
	m_textures.push_back(sf::Texture());
	m_textures.back().loadFromFile("img/brickWall.jpg");
	m_textures.push_back(sf::Texture());
	m_textures.back().loadFromFile("img/tree.jpg");
	m_textures.push_back(sf::Texture());
	m_textures.back().loadFromFile("img/water.jpg");

	// Cre un texture de rendu
	m_renderTexture.create(SCREEN_WIDTH, SCREEN_HEIGHT);
	
	// Initialise le sprite
	m_sprite.setTexture(m_renderTexture.getTexture());

	// Charge la carte et remplie la texture de rendu
	LoadTileMap();
	UpdateTexture();
}

TileMap::~TileMap(void)
{
}

void TileMap::SetTileLife(int i, int j, int life)
{
	try {
	m_map.at(i).at(j).SetLife(life);
	}
	catch(std::exception const& e) {
        std::cerr << "ERREUR : " << e.what() << std::endl;
    }
}

void TileMap::AddToTileLife(int i, int j, int life)
{
	try	{
		m_map.at(i).at(j).SetLife(m_map.at(i).at(j).GetLife() + life);
	}
	catch(std::exception const& e) {
		std::cerr << "ERREUR : " << e.what() << std::endl;
	}
}

// Charge la carte
void TileMap::LoadTileMap()
{
	// Remplissage du tableau
	sf::Color pixel;
	sf::Vector2f tank1_InitPos(0.f, 0.f), tank2_InitPos(5 * CELL_SIZE, 10 * CELL_SIZE);
	m_map.clear();
	m_map.resize(m_nbRows);
	for (unsigned int i = 0; i < m_nbRows; ++i)
	{
		m_map.at(i).insert(m_map.at(i).begin(), m_nbColumns, Tile(&m_textures.at(0)));
		for (unsigned int j = 0; j < m_nbColumns; ++j)
		{
			pixel = m_tilemap.getPixel(i, j);

			// Position des tanks
			if (pixel == sf::Color::Green)
				m_tank1_initPos = sf::Vector2f(static_cast<float>(i * CELL_SIZE), static_cast<float>(j * CELL_SIZE));
			else if (pixel == sf::Color::Yellow)
				m_tank2_initPos = sf::Vector2f(static_cast<float>(i * CELL_SIZE), static_cast<float>(j * CELL_SIZE));

			// Tilemap
			// TODO: Systme de map plus pouss
			else if (pixel == sf::Color::Red) // Mur
			{
				m_map.at(i).at(j).SetAnimTexture(&m_textures.at(1));
				m_map.at(i).at(j).SetBlocking(true);
				m_map.at(i).at(j).SetHoverable(false);
				m_map.at(i).at(j).SetBreakable(false);
			}
			else if (pixel == sf::Color::Blue) // Arbres
			{
				m_map.at(i).at(j).SetAnimTexture(&m_textures.at(2));
				m_map.at(i).at(j).SetInterval(0.42f); // Oui ! La Rponse  la Question !
				m_map.at(i).at(j).SetBlocking(true);
				m_map.at(i).at(j).SetHoverable(false);
				m_map.at(i).at(j).SetBreakable(true);
			}
			else if (pixel == sf::Color::Cyan) // Eau
			{
				m_map.at(i).at(j).SetAnimTexture(&m_textures.at(3));
				m_map.at(i).at(j).SetBlocking(true);
				m_map.at(i).at(j).SetHoverable(true);
				m_map.at(i).at(j).SetBreakable(false);
			}
			else								// Vide
			{
				m_map.at(i).at(j).SetAnimTexture(&m_textures.at(0));
				m_map.at(i).at(j).SetBlocking(false);
				m_map.at(i).at(j).SetHoverable(true);
				m_map.at(i).at(j).SetBreakable(false);
			}
			m_map.at(i).at(j).setPosition(static_cast<float>(i * CELL_SIZE), static_cast<float>(j * CELL_SIZE));
		}
	}
	UpdateTexture();
}

// Recre la RenderTexture de la carte
void TileMap::UpdateTexture()
{
	m_renderTexture.clear(sf::Color(128, 128, 128));
	for (unsigned int i = 0; i < m_map.size(); ++i)
		for (unsigned int j = 0; j < m_map.at(i).size(); ++j)
			m_renderTexture.draw(m_map.at(i).at(j));
	m_renderTexture.display();
}

// Mets  jour les animations des tiles
void TileMap::UpdateAnims()
{
	for (unsigned int i = 0; i < m_map.size(); ++i)
		for (unsigned int j = 0; j < m_map.at(i).size(); ++j)
			m_map.at(i).at(j).UpdateAnim();
}

// Mets  jour tous la RenderTexture avec les nouveaux tiles
void TileMap::UpdateTiles()
{
	for (unsigned int i = 0; i < m_map.size(); ++i)
		for (unsigned int j = 0; j < m_map.at(i).size(); ++j)
			if (m_map.at(i).at(j).UpdateAnim())
				m_renderTexture.draw(m_map.at(i).at(j));
	m_renderTexture.display();
}

// Affiche la carte
void TileMap::Draw(sf::RenderTarget& target)
{
	// Mets  jour les animations et la RenderTex si besoin
	UpdateTiles();

	// Affiche la texture
	target.draw(m_sprite);
}