/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#pragma once
#include <iostream>
#include <SFML\Graphics.hpp>
#include <string>
#include <vector>
#include "Tile.h"

class TileMap
{
public:
	TileMap(const sf::Image &tilemap);
	virtual ~TileMap(void);

	// Charge la carte
	void LoadTileMap();

	// Mets  jour les animations de tous les tiles
	void UpdateAnims();

	// Recre la RenderTexture de la carte
	void UpdateTexture();

	// Mets  jour tous la RenderTexture avec les nouveaux tiles
	void UpdateTiles();

	// Affiche la carte
	void Draw(sf::RenderTarget& target);

	// Pour modifier les dgats des tiles
	void SetTileLife(int i, int j, int life);
	void AddToTileLife(int i, int j, int life);

	// Getter
	unsigned int GetNbRow() const { return m_nbRows; }
	unsigned int GetNbColumns() const { return m_nbColumns; }
	sf::Vector2f GetTank1_InitPos() const { return m_tank1_initPos; }
	sf::Vector2f GetTank2_InitPos() const { return m_tank2_initPos; }
	const std::vector<std::vector<Tile>>& GetMap() const { return m_map; }

private:
	// Positions initiales des Tanks selon la tilemap
	sf::Vector2f m_tank1_initPos;
	sf::Vector2f m_tank2_initPos;

	// Tailles de la carte + carte
	unsigned int m_nbRows;
	unsigned int m_nbColumns;
	std::vector<std::vector<Tile>> m_map;

	// Tilemap
	const sf::Image &m_tilemap;

	// RenderTexture  afficher
	sf::Sprite m_sprite;
	sf::RenderTexture m_renderTexture;

	// Liste de textures des tiles
	std::vector<sf::Texture> m_textures;
};