/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "Client.h"

namespace Moteur
{
// Ctors
Client::Client()
{
    // Serveur
    m_Serveur = new sf::TcpSocket;
    m_Connecte = false;

    // Threads
    m_ThreadEnvoi = new Moteur::ThreadEnvoi(this);
    m_ThreadReception = new Moteur::ThreadReception(this);

    // ID du client
    m_MonID = -1;
    m_IDRecu = false;

    // Liste des paquets
    m_NouveauPaquetRecu = false;
    m_ListePaquets = new std::queue<sf::Packet>;
}
Client::Client(sf::TcpSocket* serveur)
{
    // Serveur
    m_Serveur = serveur;
    m_Connecte = true;//m_Serveur->IsValid();

    // Threads
    m_ThreadEnvoi = new Moteur::ThreadEnvoi(this);
    m_ThreadReception = new Moteur::ThreadReception(this);

    // ID du client
    m_MonID = -1;
    m_IDRecu = false;

    // Liste des paquets
    m_NouveauPaquetRecu = false;
    m_ListePaquets = new std::queue<sf::Packet>;

    // Lance le thread de rception
    m_ThreadReception->launch();
}

// Dtor
Client::~Client()
{
    // Arrte les threads
    m_ThreadReception->Arreter();
    m_ThreadReception->wait(); // Attend l'arrt des threads
    m_ThreadEnvoi->wait();

    // Supprime les pointeurs
    //delete m_ThreadReception;
    //delete m_ThreadEnvoi;
	m_Serveur->disconnect();
    //delete m_Serveur;
    delete m_ListePaquets;
}

// Se connecter  un serveur sur l'IP et le port spcifi
bool Client::Connecter(const sf::IpAddress& adresseIP, unsigned short port)
{
    // Pas la peine de vrifier si il est valide, la SFML le fait pour nous
    // Se connecte  l'adresse et au port spcifis
    if (m_Serveur->connect(adresseIP, port) != sf::Socket::Done)
    {
        std::cerr << "Erreur durant la connexion !" << std::endl;
		sf::sleep(sf::seconds(1.5f));
        exit(101);
        return false;
    }

    // Nous sommes connects
    m_Connecte = true;

    // Lance le thread de rception
    m_ThreadReception->launch();

    // Tout s'est bien pass
    return true;
}
bool Client::Connecter(const std::string& adresseIP, unsigned short port)
{
    return this->Connecter(sf::IpAddress(adresseIP), port);
}

// Se dconnecter du serveur actuel
void Client::Deconnecter()
{
    if (m_Connecte)
    {
        // Envoie d'un message au serveur comme quoi on se dconnecte
        sf::Packet paquet;
        paquet << 0 << m_MonID << 2;
        this->Envoyer(paquet, true);

        // Puis suppressions
        m_ThreadReception->terminate();
		m_Serveur->disconnect();
        m_Connecte = false;
        m_IDRecu = false;
    }
    std::cout << "La d\x82""connection au serveur a r\x82""ussie !" << std::endl;
}

// Mthode appel par le thread de rception quand il reoit un paquet
bool Client::Recevoir(sf::Packet &paquet)
{
    // Cre une copie du paquet pour pouvoir le transmettre sans les modifs apports ici
    sf::Packet paquetCopie = paquet;

    // Traite le paquet
    // Rcupre quel type d'action est  effectuer
    int action;
    paquet >> action;

    if (action == 0) // Message serveur
    {
#ifdef NET_DEBUG
        // On indique qu'il s'agit d'un message du serveur
        std::cout << "Le paquet contient un message du serveur." << std::endl;
#endif

        int contenu;
        paquet >> contenu;
        if (contenu == 0) // Liste des clients
        {
			// Copie le paquet et indique qu'il y en a un disponible
			m_ListePaquets->push(paquetCopie);
			m_NouveauPaquetRecu = true;

            // Rceptionne la liste
            int nbDeClients;
            paquet >> nbDeClients;

            // Vide la liste des clients actuelle
            m_ListeClients.clear();

            // Remplie la liste des clients
            int IDtmp = 0;
            std::string nomTmp = "";
            std::string adresseIPtmp = "";
            for (int i = 0; i < nbDeClients && !paquet.endOfPacket(); ++i)
            {
                paquet >> IDtmp; // Rcupre l'ID (le int)
                paquet >> nomTmp >> adresseIPtmp; // Puis le nom et l'adresse du client (et les strings)
                m_ListeClients.push_back(Moteur::InfosClient(IDtmp, nomTmp, adresseIPtmp));
            }
			
#ifdef NET_DEBUG
            // On saute une ligne pour faire propre
            std::cout << std::endl;
#endif

            return true;
        }
        //-------------------------------------------------------------------------------------------------
        if (contenu == 1) // ID du client (moi)
        {
            // Change l'ID
            paquet >> m_MonID;
            m_IDRecu = true;
            std::cout << "ID du client chang\x82 en " << m_MonID << ". Les envois sont d\x82""sormais possibles." << std::endl;

            // On saute une ligne pour faire propre
            std::cout << std::endl;

            return true;
        }
        return false; // contenu inconnu
    } // Fin Message serveur
    //-------------------------------------------------------------------------------------------------
    //#################################################################################################
    //-------------------------------------------------------------------------------------------------
    if (action == 1) // Message  moi uniquement
    {
        // Copie le paquet et indique qu'il y en a un disponible
        m_ListePaquets->push(paquetCopie);
        m_NouveauPaquetRecu = true;

        // On extrait les ID
        int IDExpediteur, IDDestinataire;
        paquet >> IDExpediteur >> IDDestinataire;
		
#ifdef NET_DEBUG
        // On indique qu'il s'agit d'un paquet  moi uniquement
        std::cout << "Paquet de " << IDExpediteur << " \x85 moi uniquement." << std::endl;

        // On saute une ligne pour faire propre
        std::cout << std::endl;
#endif

        return true;
    } // Fin Message  moi uniquement
    //-------------------------------------------------------------------------------------------------
    //#################################################################################################
    //-------------------------------------------------------------------------------------------------
    if (action == 2) // Message  tous les clients
    {
        // Copie le paquet et indique qu'il y en a un disponible
        m_ListePaquets->push(paquetCopie);
        m_NouveauPaquetRecu = true;

        // On extrait les ID
        int IDExpediteur;
        paquet >> IDExpediteur;
		
#ifdef NET_DEBUG
        // On indique qu'il s'agit d'un paquet  moi uniquement
        std::cout << "Paquet de " << IDExpediteur << " \x85 tous les clients." << std::endl;

        // On saute une ligne pour faire propre
        std::cout << std::endl;
#endif

        return true;
    } // Fin Message  tous les clients
	
#ifdef NET_DEBUG
    // Paquet de forme inconnu
    std::cout << "Paquet de forme inconnue re\x87""u !" << std::endl;
#endif

    return true;
}

// Mthode qui transmet un paquet  envoyer au thread d'envoi
bool Client::Envoyer(const sf::Packet &paquet, bool bloquant)
{
    // Transfert du paquet au thread d'envoi
    if (!m_Connecte || !m_IDRecu)
    {
        std::cout << "Erreur : envoie sans avoir re\x87""u l'ID -> paquet non envoy\x82 ! / sans etre connect\x82 !" << std::endl;
        return false;
    }
    else
        return m_ThreadEnvoi->Envoyer(paquet, bloquant);

    return true;
}

// Envoie $nom au serveur, selon le "langage" dfini dans messages.txt
bool Client::EnvoyerNom(const std::string& nom)
{
    // Voir messages.txt pour la composition du paquet
    sf::Packet paquetTmp;
    paquetTmp << 0 << m_MonID << 1 << nom;

    // Transfert du paquet au thread d'envoi
    return m_ThreadEnvoi->Envoyer(paquetTmp);
}
} // Fin du namespace Moteur
