/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include <SFML/Network.hpp>
#include <string>
#include <vector>
#include <iostream>

#ifndef CLIENT_H
#define CLIENT_H

#include "ThreadEnvoi.h"
#include "ThreadReception.h"
#include "InfosClient.h"

namespace Moteur
{
    class Client
    {
    public:
        // Ctor & dtor
        Client();
		Client(sf::TcpSocket* serveur);
        virtual ~Client();

        // Fonctions
        bool Connecter(const std::string& adresseIP, unsigned short port);
        bool Connecter(const sf::IpAddress& adresseIP, unsigned short port);
        void Deconnecter();
        bool Recevoir(sf::Packet &paquet);
        bool Envoyer(const sf::Packet &paquet, bool bloquant = false);
        bool EnvoyerNom(const std::string& nom);

        // Fonctions "Is"
        bool IsNouveauPaquetRecu() const { return m_NouveauPaquetRecu; }
        bool IsConnecte() const { return m_Connecte; }
        bool IsIDRecu() const { return m_IDRecu; }

        // Fonctions "Get"
        int GetID() const { return m_MonID; }
        sf::Packet& GetPaquet() { return m_ListePaquets->back(); }
        sf::TcpSocket& GetServeur() const { return *m_Serveur; }

        // Fonctions "Set"
        void SetPaquetLu() { m_NouveauPaquetRecu = false; }
        void SetPaquetNonLu() { m_NouveauPaquetRecu = true; }

    protected:

    private:
        // Liste des paquets
        bool m_NouveauPaquetRecu;
        std::queue<sf::Packet>* m_ListePaquets;

        // Serveur
        bool m_IDRecu; // Savoir si l'ID du client est reu.
        sf::TcpSocket* m_Serveur;
        bool m_Connecte; // Connect  un serveur

        // Liste des clients
        int m_MonID; // ID du client (moi)
        std::vector<InfosClient> m_ListeClients;

        // Threads
        Moteur::ThreadEnvoi* m_ThreadEnvoi;
        Moteur::ThreadReception* m_ThreadReception;
    };
} // Fin du namespace Moteur

#endif // CLIENT_H
