/*
** Copyright (C) 2012 Lacroix Thomas
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU Lesser General Public
** License as published by the Free Software Foundation; either
** version 2.1 of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** Lesser General Public License for more details.
** 
** You should have received a copy of the GNU Lesser General Public
** License along with this program; if not, write to the Free Software
** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

#include "ThreadReception.h"

namespace Moteur
{
// Ctors
ThreadReception::ThreadReception(Moteur::Client* client)
{
    m_Client = client;
    m_Arreter = false;
}

// Dtor
ThreadReception::~ThreadReception()
{
    delete m_Client;
}

// Demande au thread de s'arrter
void ThreadReception::Arreter()
{
    m_Arreter = true;
}

// Boucle infinie de rception qui renvoie chaque paquet au thread du client
void ThreadReception::Run()
{
    // Boucle infinie de rception -> tant que connect && ne doit pas s'arrter
    while (m_Client->IsConnecte() && !m_Arreter)
    {
        sf::Packet Paquet;
        sf::Socket::Status Statut;

        // Rception du paquet
        Statut = m_Client->GetServeur().receive(Paquet); // La rception se fait  cette ligne
        if (Statut == sf::Socket::Done)
        {
#ifdef NET_DEBUG
            // Paquet correctement reu !
            std::cout << "Paquet correctement re\x87u !" << std::endl;
#endif

            // On transmet le paquet au thread du client
            if (m_Client->Recevoir(Paquet))
            {
#ifdef NET_DEBUG
                std::cout << "Paquet correctement transmis au thread du client !" << std::endl;
#endif
            }
            else
            {
                std::cout << "La transmission du paquet au thread de client a \x82""chou\x82" << std::endl;
            }
        }
        else if (Statut == sf::Socket::Disconnected)
        {
            std::cout << "D\x82""connect\x82 du serveur !" << std::endl;
            break;
        }
        else if (Statut == sf::Socket::NotReady)
        {
            std::cout << "Time out !" << std::endl;
        }
        else exit(102);
    }
#ifdef NET_DEBUG
    // Si il ne s'arrte pas volontairement
    if (!m_Arreter)
        std::cout << "Le thread de r\x82""ception s'est stopp\x82, il s'agit probablement d'un bug !" << std::endl;
#endif
}

} // Fin du namespace Moteur
